@@ -2,11 +2,12 @@ package controller
 
 import (
 	"context"
+	"encoding/json"
 	"fmt"
 	"net/url"
 	"reflect"
+	"regexp"
 	"strconv"
-	"strings"
 	"time"
 
 	sdkapi "kubevirt.io/controller-lifecycle-operator-sdk/pkg/sdk/api"
@@ -92,9 +93,10 @@ const (
 
 	// ImportTargetInUse is reason for event created when an import pvc is in use
 	ImportTargetInUse = "ImportTargetInUse"
+)
 
-	// PreallocationApplied is a string inserted into importer's/uploader's exit message
-	PreallocationApplied = "Preallocation applied"
+var (
+	vddkInfoMatch = regexp.MustCompile(`((.*; )|^)VDDK: (?P<info>{.*})`)
 )
 
 // ImportReconciler members
@@ -128,6 +130,10 @@ type importPodEnvVar struct {
 	previousCheckpoint string
 	finalCheckpoint    string
 	preallocation      bool
+	httpProxy          string
+	httpsProxy         string
+	noProxy            string
+	certConfigMapProxy string
 }
 
 // NewImportController creates a new instance of the import controller.
@@ -175,11 +181,10 @@ func addImportControllerWatches(mgr manager.Manager, importController controller
 	return nil
 }
 
-func shouldReconcilePVC(pvc *corev1.PersistentVolumeClaim,
-	isImmediateBindingRequested bool,
-	featureGates featuregates.FeatureGates,
+func (r *ImportReconciler) shouldReconcilePVC(pvc *corev1.PersistentVolumeClaim,
 	log logr.Logger) (bool, error) {
-	waitForFirstConsumerEnabled, err := isWaitForFirstConsumerEnabled(isImmediateBindingRequested, featureGates)
+	_, isImmediateBindingRequested := pvc.Annotations[AnnImmediateBinding]
+	waitForFirstConsumerEnabled, err := isWaitForFirstConsumerEnabled(isImmediateBindingRequested, r.featureGates)
 
 	if err != nil {
 		return false, err
@@ -196,7 +201,7 @@ func isPVCComplete(pvc *corev1.PersistentVolumeClaim) bool {
 }
 
 // Reconcile the reconcile loop for the CDIConfig object.
-func (r *ImportReconciler) Reconcile(req reconcile.Request) (reconcile.Result, error) {
+func (r *ImportReconciler) Reconcile(_ context.Context, req reconcile.Request) (reconcile.Result, error) {
 	log := r.log.WithValues("PVC", req.NamespacedName)
 	log.V(1).Info("reconciling Import PVCs")
 
@@ -208,9 +213,8 @@ func (r *ImportReconciler) Reconcile(req reconcile.Request) (reconcile.Result, e
 		}
 		return reconcile.Result{}, err
 	}
-	_, isImmediateBindingRequested := pvc.Annotations[AnnImmediateBinding]
 
-	shouldReconcile, err := shouldReconcilePVC(pvc, isImmediateBindingRequested, r.featureGates, log)
+	shouldReconcile, err := r.shouldReconcilePVC(pvc, log)
 	if err != nil {
 		return reconcile.Result{}, err
 	}
@@ -270,7 +274,7 @@ func (r *ImportReconciler) reconcilePvc(pvc *corev1.PersistentVolumeClaim, log l
 			// Don't create the POD if the PVC is completed already
 			log.V(1).Info("PVC is already complete")
 		} else if pvc.DeletionTimestamp == nil {
-			podsUsingPVC, err := getPodsUsingPVCs(r.client, pvc.Namespace, sets.NewString(pvc.Name), false)
+			podsUsingPVC, err := GetPodsUsingPVCs(r.client, pvc.Namespace, sets.NewString(pvc.Name), false)
 			if err != nil {
 				return reconcile.Result{}, err
 			}
@@ -367,8 +371,11 @@ func (r *ImportReconciler) updatePvcFromPod(pvc *corev1.PersistentVolumeClaim, p
 	if pod.Status.ContainerStatuses != nil &&
 		pod.Status.ContainerStatuses[0].State.Terminated != nil &&
 		pod.Status.ContainerStatuses[0].State.Terminated.ExitCode == 0 {
-		if strings.Contains(pod.Status.ContainerStatuses[0].State.Terminated.Message, PreallocationApplied) {
-			anno[AnnPreallocationApplied] = "true"
+	}
+	if pod.Status.ContainerStatuses != nil &&
+		pod.Status.ContainerStatuses[0].State.Terminated != nil {
+		if getSource(pvc) == SourceVDDK {
+			r.saveVddkAnnotations(pvc, pod, log)
 		}
 	}
 
@@ -427,6 +434,31 @@ func (r *ImportReconciler) updatePvcFromPod(pvc *corev1.PersistentVolumeClaim, p
 	return nil
 }
 
+func (r *ImportReconciler) saveVddkAnnotations(pvc *corev1.PersistentVolumeClaim, pod *corev1.Pod, log logr.Logger) {
+	terminationMessage := pod.Status.ContainerStatuses[0].State.Terminated.Message
+	log.V(1).Info("Saving VDDK annotations from pod status message: ", "message", terminationMessage)
+
+	var terminationInfo string
+	matches := vddkInfoMatch.FindAllStringSubmatch(terminationMessage, -1)
+	for index, matchName := range vddkInfoMatch.SubexpNames() {
+		if matchName == "info" && len(matches) > 0 {
+			terminationInfo = matches[0][index]
+			break
+		}
+	}
+
+	var vddkInfo util.VddkInfo
+	err := json.Unmarshal([]byte(terminationInfo), &vddkInfo)
+	if err == nil {
+		if vddkInfo.Host != "" {
+			pvc.Annotations[AnnVddkHostConnection] = vddkInfo.Host
+		}
+		if vddkInfo.Version != "" {
+			pvc.Annotations[AnnVddkVersion] = vddkInfo.Version
+		}
+	}
+}
+
 func (r *ImportReconciler) updatePVC(pvc *corev1.PersistentVolumeClaim, log logr.Logger) error {
 	log.V(1).Info("Annotations are now", "pvc.anno", pvc.GetAnnotations())
 	if err := r.client.Update(context.TODO(), pvc); err != nil {
@@ -466,9 +498,8 @@ func (r *ImportReconciler) createImporterPod(pvc *corev1.PersistentVolumeClaim)
 	if err != nil {
 		return err
 	}
-
 	// all checks passed, let's create the importer pod!
-	pod, err := createImporterPod(r.log, r.client, r.image, r.verbose, r.pullPolicy, podEnvVar, pvc, scratchPvcName, vddkImageName)
+	pod, err := createImporterPod(r.log, r.client, r.image, r.verbose, r.pullPolicy, podEnvVar, pvc, scratchPvcName, vddkImageName, getPriorityClass(pvc))
 
 	if err != nil {
 		return err
@@ -485,7 +516,7 @@ func (r *ImportReconciler) createImporterPod(pvc *corev1.PersistentVolumeClaim)
 func (r *ImportReconciler) createImportEnvVar(pvc *corev1.PersistentVolumeClaim) (*importPodEnvVar, error) {
 	podEnvVar := &importPodEnvVar{}
 	podEnvVar.source = getSource(pvc)
-	podEnvVar.contentType = getContentType(pvc)
+	podEnvVar.contentType = GetContentType(pvc)
 
 	var err error
 	if podEnvVar.source != SourceNone {
@@ -497,11 +528,14 @@ func (r *ImportReconciler) createImportEnvVar(pvc *corev1.PersistentVolumeClaim)
 		if podEnvVar.secretName == "" {
 			r.log.V(2).Info("no secret will be supplied to endpoint", "endPoint", podEnvVar.ep)
 		}
+		//get the CDIConfig to extract the proxy configuration to be used to import an image
+		cdiConfig := &cdiv1.CDIConfig{}
+		r.client.Get(context.TODO(), types.NamespacedName{Name: common.ConfigName}, cdiConfig)
 		podEnvVar.certConfigMap, err = r.getCertConfigMap(pvc)
 		if err != nil {
 			return nil, err
 		}
-		podEnvVar.insecureTLS, err = r.isInsecureTLS(pvc)
+		podEnvVar.insecureTLS, err = r.isInsecureTLS(pvc, cdiConfig)
 		if err != nil {
 			return nil, err
 		}
@@ -512,6 +546,24 @@ func (r *ImportReconciler) createImportEnvVar(pvc *corev1.PersistentVolumeClaim)
 		podEnvVar.previousCheckpoint = getValueFromAnnotation(pvc, AnnPreviousCheckpoint)
 		podEnvVar.currentCheckpoint = getValueFromAnnotation(pvc, AnnCurrentCheckpoint)
 		podEnvVar.finalCheckpoint = getValueFromAnnotation(pvc, AnnFinalCheckpoint)
+
+		var field string
+		if field, err = GetImportProxyConfig(cdiConfig, common.ImportProxyHTTP); err != nil {
+			r.log.V(3).Info("no proxy http url will be supplied:", err.Error())
+		}
+		podEnvVar.httpProxy = field
+		if field, err = GetImportProxyConfig(cdiConfig, common.ImportProxyHTTPS); err != nil {
+			r.log.V(3).Info("no proxy https url will be supplied:", err.Error())
+		}
+		podEnvVar.httpsProxy = field
+		if field, err = GetImportProxyConfig(cdiConfig, common.ImportProxyNoProxy); err != nil {
+			r.log.V(3).Info("the noProxy field will not be supplied:", err.Error())
+		}
+		podEnvVar.noProxy = field
+		if field, err = GetImportProxyConfig(cdiConfig, common.ImportProxyConfigMapName); err != nil {
+			r.log.V(3).Info("no proxy CA certiticate will be supplied:", err.Error())
+		}
+		podEnvVar.certConfigMapProxy = field
 	}
 
 	fsOverhead, err := GetFilesystemOverhead(r.client, pvc)
@@ -532,9 +584,7 @@ func (r *ImportReconciler) createImportEnvVar(pvc *corev1.PersistentVolumeClaim)
 	return podEnvVar, nil
 }
 
-func (r *ImportReconciler) isInsecureTLS(pvc *corev1.PersistentVolumeClaim) (bool, error) {
-	var configMapName string
-
+func (r *ImportReconciler) isInsecureTLS(pvc *corev1.PersistentVolumeClaim, cdiConfig *cdiv1.CDIConfig) (bool, error) {
 	value, ok := pvc.Annotations[AnnEndpoint]
 	if !ok || value == "" {
 		return false, nil
@@ -545,13 +595,19 @@ func (r *ImportReconciler) isInsecureTLS(pvc *corev1.PersistentVolumeClaim) (boo
 		return false, err
 	}
 
-	switch url.Scheme {
-	case "docker":
-		configMapName = common.InsecureRegistryConfigMap
-	default:
+	if url.Scheme != "docker" {
 		return false, nil
 	}
 
+	for _, value := range cdiConfig.Spec.InsecureRegistries {
+		r.log.V(1).Info("Checking host against value", "host", url.Host, "value", value)
+		if value == url.Host {
+			return true, nil
+		}
+	}
+
+	// ConfigMap is obsoleted and supported only for upgrade. It won't be refered anymore by future releases.
+	configMapName := common.InsecureRegistryConfigMap
 	r.log.V(1).Info("Checking configmap for host", "configMapName", configMapName, "host URL", url.Host)
 
 	cm := &corev1.ConfigMap{}
@@ -563,9 +619,8 @@ func (r *ImportReconciler) isInsecureTLS(pvc *corev1.PersistentVolumeClaim) (boo
 		return false, err
 	}
 
-	for key, value := range cm.Data {
-		r.log.V(1).Info("Checking host against key, value pair", "host", url.Host, "Key", key, "Value", value)
-
+	for _, value := range cm.Data {
+		r.log.V(1).Info("Checking host against value", "host", url.Host, "value", value)
 		if value == url.Host {
 			return true, nil
 		}
@@ -614,7 +669,7 @@ func (r *ImportReconciler) getSecretName(pvc *corev1.PersistentVolumeClaim) stri
 
 func (r *ImportReconciler) requiresScratchSpace(pvc *corev1.PersistentVolumeClaim) bool {
 	scratchRequired := false
-	contentType := getContentType(pvc)
+	contentType := GetContentType(pvc)
 	// All archive requires scratch space.
 	if contentType == "archive" {
 		scratchRequired = true
@@ -704,11 +759,11 @@ func getSource(pvc *corev1.PersistentVolumeClaim) string {
 	return source
 }
 
-// returns the source string which determines the type of source. If no source or invalid source found, default to http
-func getContentType(pvc *corev1.PersistentVolumeClaim) string {
+// GetContentType returns the content type of the source image. If invalid or not set, default to kubevirt
+func GetContentType(pvc *corev1.PersistentVolumeClaim) string {
 	contentType, found := pvc.Annotations[AnnContentType]
 	if !found {
-		contentType = ""
+		return string(cdiv1.DataVolumeKubeVirt)
 	}
 	switch contentType {
 	case
@@ -756,7 +811,7 @@ func createImportPodNameFromPvc(pvc *corev1.PersistentVolumeClaim) string {
 // createImporterPod creates and returns a pointer to a pod which is created based on the passed-in endpoint, secret
 // name, and pvc. A nil secret means the endpoint credentials are not passed to the
 // importer pod.
-func createImporterPod(log logr.Logger, client client.Client, image, verbose, pullPolicy string, podEnvVar *importPodEnvVar, pvc *corev1.PersistentVolumeClaim, scratchPvcName *string, vddkImageName *string) (*corev1.Pod, error) {
+func createImporterPod(log logr.Logger, client client.Client, image, verbose, pullPolicy string, podEnvVar *importPodEnvVar, pvc *corev1.PersistentVolumeClaim, scratchPvcName *string, vddkImageName *string, priorityClassName string) (*corev1.Pod, error) {
 	podResourceRequirements, err := GetDefaultPodResourceRequirements(client)
 	if err != nil {
 		return nil, err
@@ -767,7 +822,7 @@ func createImporterPod(log logr.Logger, client client.Client, image, verbose, pu
 		return nil, err
 	}
 
-	pod := makeImporterPodSpec(pvc.Namespace, image, verbose, pullPolicy, podEnvVar, pvc, scratchPvcName, podResourceRequirements, workloadNodePlacement, vddkImageName)
+	pod := makeImporterPodSpec(pvc.Namespace, image, verbose, pullPolicy, podEnvVar, pvc, scratchPvcName, podResourceRequirements, workloadNodePlacement, vddkImageName, priorityClassName)
 
 	if err := client.Create(context.TODO(), pod); err != nil {
 		return nil, err
@@ -777,7 +832,7 @@ func createImporterPod(log logr.Logger, client client.Client, image, verbose, pu
 }
 
 // makeImporterPodSpec creates and return the importer pod spec based on the passed-in endpoint, secret and pvc.
-func makeImporterPodSpec(namespace, image, verbose, pullPolicy string, podEnvVar *importPodEnvVar, pvc *corev1.PersistentVolumeClaim, scratchPvcName *string, podResourceRequirements *corev1.ResourceRequirements, workloadNodePlacement *sdkapi.NodePlacement, vddkImageName *string) *corev1.Pod {
+func makeImporterPodSpec(namespace, image, verbose, pullPolicy string, podEnvVar *importPodEnvVar, pvc *corev1.PersistentVolumeClaim, scratchPvcName *string, podResourceRequirements *corev1.ResourceRequirements, workloadNodePlacement *sdkapi.NodePlacement, vddkImageName *string, priorityClassName string) *corev1.Pod {
 	// importer pod name contains the pvc name
 	podName, _ := pvc.Annotations[AnnImportPod]
 
@@ -851,11 +906,12 @@ func makeImporterPodSpec(namespace, image, verbose, pullPolicy string, podEnvVar
 					},
 				},
 			},
-			RestartPolicy: corev1.RestartPolicyOnFailure,
-			Volumes:       volumes,
-			NodeSelector:  workloadNodePlacement.NodeSelector,
-			Tolerations:   workloadNodePlacement.Tolerations,
-			Affinity:      workloadNodePlacement.Affinity,
+			RestartPolicy:     corev1.RestartPolicyOnFailure,
+			Volumes:           volumes,
+			NodeSelector:      workloadNodePlacement.NodeSelector,
+			Tolerations:       workloadNodePlacement.Tolerations,
+			Affinity:          workloadNodePlacement.Affinity,
+			PriorityClassName: priorityClassName,
 		},
 	}
 
@@ -930,6 +986,15 @@ func makeImporterPodSpec(namespace, image, verbose, pullPolicy string, podEnvVar
 		pod.Spec.Volumes = append(pod.Spec.Volumes, vol)
 	}
 
+	if podEnvVar.certConfigMapProxy != "" {
+		vm := corev1.VolumeMount{
+			Name:      ProxyCertVolName,
+			MountPath: common.ImporterProxyCertDir,
+		}
+		pod.Spec.Containers[0].VolumeMounts = append(pod.Spec.Containers[0].VolumeMounts, vm)
+		pod.Spec.Volumes = append(pod.Spec.Volumes, createProxyConfigMapVolume(CertVolName, podEnvVar.certConfigMapProxy))
+	}
+
 	if podEnvVar.contentType == string(cdiv1.DataVolumeKubeVirt) {
 		// Set the fsGroup on the security context to the QemuSubGid
 		if pod.Spec.SecurityContext == nil {
@@ -942,6 +1007,19 @@ func makeImporterPodSpec(namespace, image, verbose, pullPolicy string, podEnvVar
 	return pod
 }
 
+func createProxyConfigMapVolume(certVolName, objRef string) corev1.Volume {
+	return corev1.Volume{
+		Name: CertVolName,
+		VolumeSource: corev1.VolumeSource{
+			ConfigMap: &corev1.ConfigMapVolumeSource{
+				LocalObjectReference: corev1.LocalObjectReference{
+					Name: objRef,
+				},
+			},
+		},
+	}
+}
+
 // this is being called for pods using PV with filesystem volume mode
 func addImportVolumeMounts() []corev1.VolumeMount {
 	volumeMounts := []corev1.VolumeMount{
@@ -1000,6 +1078,18 @@ func makeImportEnv(podEnvVar *importPodEnvVar, uid types.UID) []corev1.EnvVar {
 			Name:  common.ImporterThumbprint,
 			Value: podEnvVar.thumbprint,
 		},
+		{
+			Name:  common.ImportProxyHTTP,
+			Value: podEnvVar.httpProxy,
+		},
+		{
+			Name:  common.ImportProxyHTTPS,
+			Value: podEnvVar.httpsProxy,
+		},
+		{
+			Name:  common.ImportProxyNoProxy,
+			Value: podEnvVar.noProxy,
+		},
 		{
 			Name:  common.ImporterCurrentCheckpoint,
 			Value: podEnvVar.currentCheckpoint,
@@ -1047,5 +1137,11 @@ func makeImportEnv(podEnvVar *importPodEnvVar, uid types.UID) []corev1.EnvVar {
 			Value: common.ImporterCertDir,
 		})
 	}
+	if podEnvVar.certConfigMapProxy != "" {
+		env = append(env, corev1.EnvVar{
+			Name:  common.ImporterProxyCertDirVar,
+			Value: common.ImporterProxyCertDir,
+		})
+	}
 	return env
 }
