@@ -10,10 +10,14 @@ import (
 	"time"
 
 	. "github.com/onsi/ginkgo"
+	. "github.com/onsi/ginkgo/extensions/table"
 	. "github.com/onsi/gomega"
 
+	snapshotv1 "github.com/kubernetes-csi/external-snapshotter/v2/pkg/apis/volumesnapshot/v1beta1"
 	v1 "k8s.io/api/core/v1"
+	"k8s.io/apimachinery/pkg/api/resource"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
+	"sigs.k8s.io/controller-runtime/pkg/client"
 
 	cdiv1 "kubevirt.io/containerized-data-importer/pkg/apis/core/v1beta1"
 	"kubevirt.io/containerized-data-importer/pkg/common"
@@ -36,6 +40,7 @@ const (
 	cloneCompleteTimeout             = 270 * time.Second
 	verifyPodDeletedTimeout          = 270 * time.Second
 	controllerSkipPVCCompleteTimeout = 270 * time.Second
+	crossVolumeModeCloneMD5NumBytes  = 100000
 )
 
 var _ = Describe("all clone tests", func() {
@@ -58,21 +63,15 @@ var _ = Describe("all clone tests", func() {
 			}
 		})
 
-		It("[test_id:1354]Should clone data within same namespace", func() {
-			smartApplicable := f.IsSnapshotStorageClassAvailable()
-			sc, err := f.K8sClient.StorageV1().StorageClasses().Get(context.TODO(), f.SnapshotSCName, metav1.GetOptions{})
-			if err == nil {
-				value, ok := sc.Annotations["storageclass.kubernetes.io/is-default-class"]
-				if smartApplicable && ok && strings.Compare(value, "true") == 0 {
-					Skip("Cannot test host assisted cloning for within namespace when all pvcs are smart clone capable.")
-				}
-			}
-
+		DescribeTable("[test_id:1354]Should clone data within same namespace", func(targetSize string) {
 			pvcDef := utils.NewPVCDefinition(sourcePVCName, "1G", nil, nil)
 			pvcDef.Namespace = f.Namespace.Name
 			sourcePvc = f.CreateAndPopulateSourcePVC(pvcDef, sourcePodFillerName, fillCommand+testFile+"; chmod 660 "+testBaseDir+testFile)
-			doFileBasedCloneTest(f, pvcDef, f.Namespace, "target-dv")
-		})
+			doFileBasedCloneTest(f, pvcDef, f.Namespace, "target-dv", targetSize)
+		},
+			Entry("with same target size", "1G"),
+			Entry("with larger target", "2G"),
+		)
 
 		It("[test_id:4953]Should clone imported data within same namespace and preserve fsGroup", func() {
 			diskImagePath := filepath.Join(testBaseDir, testFile)
@@ -83,6 +82,7 @@ var _ = Describe("all clone tests", func() {
 			f.ForceBindPvcIfDvIsWaitForFirstConsumer(dataVolume)
 
 			pvc, err := f.K8sClient.CoreV1().PersistentVolumeClaims(dataVolume.Namespace).Get(context.TODO(), dataVolume.Name, metav1.GetOptions{})
+			Expect(err).ToNot(HaveOccurred())
 
 			// Create targetPvc in new NS.
 			targetDV := utils.NewCloningDataVolume("target-dv", "1Gi", pvc)
@@ -108,7 +108,7 @@ var _ = Describe("all clone tests", func() {
 			completeClone(f, f.Namespace, targetPvc, diskImagePath, sourceMD5, sourcePvcDiskGroup)
 		})
 
-		It("[test_id:1355]Should clone data across different namespaces", func() {
+		DescribeTable("[test_id:1355]Should clone data across different namespaces", func(targetSize string) {
 			pvcDef := utils.NewPVCDefinition(sourcePVCName, "1G", nil, nil)
 			pvcDef.Namespace = f.Namespace.Name
 			sourcePvc = f.CreateAndPopulateSourcePVC(pvcDef, sourcePodFillerName, fillCommand+testFile+"; chmod 660 "+testBaseDir+testFile)
@@ -117,8 +117,11 @@ var _ = Describe("all clone tests", func() {
 			})
 			Expect(err).NotTo(HaveOccurred())
 			f.AddNamespaceToDelete(targetNs)
-			doFileBasedCloneTest(f, pvcDef, targetNs, "target-dv")
-		})
+			doFileBasedCloneTest(f, pvcDef, targetNs, "target-dv", targetSize)
+		},
+			Entry("with same target size", "1G"),
+			Entry("with bigger target size", "2G"),
+		)
 
 		It("[test_id:4954]Should clone data across different namespaces when source initially in use", func() {
 			pvcDef := utils.NewPVCDefinition(sourcePVCName, "1G", nil, nil)
@@ -138,7 +141,7 @@ var _ = Describe("all clone tests", func() {
 			cloneOfAnnoExistenceTest(f, f.Namespace.Name)
 		})
 
-		It("[posneg:negative][test_id:3617]Should clone across nodes when multiple local volumes exist,", func() {
+		It("[posneg:negative][test_id:3617]Should clone across nodes when multiple local filesystem volumes exist,", func() {
 			// Get nodes, need at least 2
 			nodeList, err := f.K8sClient.CoreV1().Nodes().List(context.TODO(), metav1.ListOptions{})
 			Expect(err).ToNot(HaveOccurred())
@@ -147,7 +150,7 @@ var _ = Describe("all clone tests", func() {
 			}
 			nodeMap := make(map[string]bool)
 			for _, node := range nodeList.Items {
-				if ok, _ := nodeMap[node.Name]; !ok {
+				if ok := nodeMap[node.Name]; !ok {
 					nodeMap[node.Name] = true
 				}
 			}
@@ -159,8 +162,8 @@ var _ = Describe("all clone tests", func() {
 			// Verify we have PVs to at least 2 nodes.
 			pvNodeNames := make(map[string]int)
 			for _, pv := range pvList.Items {
-				if pv.Spec.NodeAffinity == nil || pv.Spec.NodeAffinity.Required == nil || len(pv.Spec.NodeAffinity.Required.NodeSelectorTerms) == 0 {
-					// Not a local volume PV
+				if pv.Spec.NodeAffinity == nil || pv.Spec.NodeAffinity.Required == nil || len(pv.Spec.NodeAffinity.Required.NodeSelectorTerms) == 0 || (pv.Spec.VolumeMode != nil && *pv.Spec.VolumeMode == v1.PersistentVolumeBlock) {
+					// Not a local volume filesystem PV
 					continue
 				}
 				pvNode := pv.Spec.NodeAffinity.Required.NodeSelectorTerms[0].MatchExpressions[0].Values[0]
@@ -174,7 +177,7 @@ var _ = Describe("all clone tests", func() {
 				}
 			}
 			if len(pvNodeNames) < 2 {
-				Skip("Need PVs on at least 2 nodes to test")
+				Skip("Need filesystem PVs on at least 2 nodes to test")
 			}
 
 			// Find the source and target PVs so we can label them.
@@ -250,6 +253,90 @@ var _ = Describe("all clone tests", func() {
 			utils.WaitForPersistentVolumeClaimPhase(f.K8sClient, f.Namespace.Name, v1.ClaimBound, targetPvc.Name)
 			completeClone(f, f.Namespace, targetPvc, filepath.Join(testBaseDir, testFile), fillDataFSMD5sum, "")
 		})
+		It("[test_id:cnv-5699]Should clone data from filesystem to block", func() {
+			if !f.IsBlockVolumeStorageClassAvailable() {
+				Skip("Storage Class for block volume is not available")
+			}
+			dataVolume := utils.NewDataVolumeWithHTTPImport(dataVolumeName, "1Gi", fmt.Sprintf(utils.TinyCoreIsoURL, f.CdiInstallNs))
+			dataVolume, err := utils.CreateDataVolumeFromDefinition(f.CdiClient, f.Namespace.Name, dataVolume)
+			Expect(err).ToNot(HaveOccurred())
+			f.ForceBindPvcIfDvIsWaitForFirstConsumer(dataVolume)
+			sourcePvc, err := f.K8sClient.CoreV1().PersistentVolumeClaims(dataVolume.Namespace).Get(context.TODO(), dataVolume.Name, metav1.GetOptions{})
+			Expect(err).ToNot(HaveOccurred())
+
+			targetDV := utils.NewDataVolumeCloneToBlockPV("target-dv", "1Gi", sourcePvc.Namespace, sourcePvc.Name, f.BlockSCName)
+			tagretDataVolume, err := utils.CreateDataVolumeFromDefinition(f.CdiClient, f.Namespace.Name, targetDV)
+			Expect(err).ToNot(HaveOccurred())
+			targetPvc, err := utils.WaitForPVC(f.K8sClient, tagretDataVolume.Namespace, tagretDataVolume.Name)
+			Expect(err).ToNot(HaveOccurred())
+
+			By("Wait for target PVC Bound phase")
+			utils.WaitForPersistentVolumeClaimPhase(f.K8sClient, f.Namespace.Name, v1.ClaimBound, targetPvc.Name)
+			By("Wait for target DV Succeeded phase")
+			err = utils.WaitForDataVolumePhaseWithTimeout(f.CdiClient, f.Namespace.Name, cdiv1.Succeeded, "target-dv", cloneCompleteTimeout)
+			Expect(err).ToNot(HaveOccurred())
+
+			By("Source file system pvc md5summing")
+			diskImagePath := filepath.Join(testBaseDir, testFile)
+			sourceMD5, err := f.GetMD5(f.Namespace, sourcePvc, diskImagePath, crossVolumeModeCloneMD5NumBytes)
+			Expect(err).ToNot(HaveOccurred())
+			By("Deleting verifier pod")
+			err = utils.DeleteVerifierPod(f.K8sClient, f.Namespace.Name)
+			Expect(err).ToNot(HaveOccurred())
+			_, err = utils.WaitPodDeleted(f.K8sClient, utils.VerifierPodName, f.Namespace.Name, verifyPodDeletedTimeout)
+			Expect(err).ToNot(HaveOccurred())
+
+			By("Target block pvc md5summing")
+			targetMD5, err := f.GetMD5(f.Namespace, targetPvc, testBaseDir, crossVolumeModeCloneMD5NumBytes)
+			Expect(err).ToNot(HaveOccurred())
+			Expect(sourceMD5 == targetMD5).To(BeTrue())
+			By("Deleting verifier pod")
+			err = utils.DeleteVerifierPod(f.K8sClient, f.Namespace.Name)
+			Expect(err).ToNot(HaveOccurred())
+		})
+		It("[test_id:cnv-5970]Should clone data from block to filesystem", func() {
+			if !f.IsBlockVolumeStorageClassAvailable() {
+				Skip("Storage Class for block volume is not available")
+			}
+			dataVolume := utils.NewDataVolumeWithHTTPImportToBlockPV(dataVolumeName, "1Gi", fmt.Sprintf(utils.TinyCoreIsoURL, f.CdiInstallNs), f.BlockSCName)
+			dataVolume, err := utils.CreateDataVolumeFromDefinition(f.CdiClient, f.Namespace.Name, dataVolume)
+			Expect(err).ToNot(HaveOccurred())
+			f.ForceBindPvcIfDvIsWaitForFirstConsumer(dataVolume)
+			sourcePvc, err := f.K8sClient.CoreV1().PersistentVolumeClaims(dataVolume.Namespace).Get(context.TODO(), dataVolume.Name, metav1.GetOptions{})
+			Expect(err).ToNot(HaveOccurred())
+
+			volumeMode := v1.PersistentVolumeMode(v1.PersistentVolumeFilesystem)
+			targetDV := utils.NewDataVolumeForImageCloning("target-dv", "2Gi", sourcePvc.Namespace, sourcePvc.Name, nil, &volumeMode)
+			targetDataVolume, err := utils.CreateDataVolumeFromDefinition(f.CdiClient, f.Namespace.Name, targetDV)
+			Expect(err).ToNot(HaveOccurred())
+			f.ForceBindPvcIfDvIsWaitForFirstConsumer(targetDataVolume)
+			targetPvc, err := utils.WaitForPVC(f.K8sClient, targetDataVolume.Namespace, targetDataVolume.Name)
+			Expect(err).ToNot(HaveOccurred())
+
+			By("Wait for target PVC Bound phase")
+			utils.WaitForPersistentVolumeClaimPhase(f.K8sClient, f.Namespace.Name, v1.ClaimBound, targetPvc.Name)
+			By("Wait for target DV Succeeded phase")
+			err = utils.WaitForDataVolumePhaseWithTimeout(f.CdiClient, f.Namespace.Name, cdiv1.Succeeded, "target-dv", cloneCompleteTimeout)
+			Expect(err).ToNot(HaveOccurred())
+
+			By("Source block pvc md5summing")
+			sourceMD5, err := f.GetMD5(f.Namespace, sourcePvc, testBaseDir, crossVolumeModeCloneMD5NumBytes)
+			Expect(err).ToNot(HaveOccurred())
+			By("Deleting verifier pod")
+			err = utils.DeleteVerifierPod(f.K8sClient, f.Namespace.Name)
+			Expect(err).ToNot(HaveOccurred())
+			_, err = utils.WaitPodDeleted(f.K8sClient, utils.VerifierPodName, f.Namespace.Name, verifyPodDeletedTimeout)
+			Expect(err).ToNot(HaveOccurred())
+
+			By("Target file system pvc md5summing")
+			diskImagePath := filepath.Join(testBaseDir, testFile)
+			targetMD5, err := f.GetMD5(f.Namespace, targetPvc, diskImagePath, crossVolumeModeCloneMD5NumBytes)
+			Expect(err).ToNot(HaveOccurred())
+			Expect(sourceMD5 == targetMD5).To(BeTrue())
+			By("Deleting verifier pod")
+			err = utils.DeleteVerifierPod(f.K8sClient, f.Namespace.Name)
+			Expect(err).ToNot(HaveOccurred())
+		})
 	})
 
 	var _ = Describe("Validate creating multiple clones of same source Data Volume", func() {
@@ -264,6 +351,9 @@ var _ = Describe("all clone tests", func() {
 		AfterEach(func() {
 			dvs := []*cdiv1.DataVolume{sourceDv, targetDv1, targetDv2, targetDv3}
 			for _, dv := range dvs {
+				if dv != nil && dv.Status.Phase == cdiv1.Succeeded {
+					validateCloneType(f, dv)
+				}
 				cleanDv(f, dv)
 			}
 		})
@@ -384,6 +474,9 @@ var _ = Describe("all clone tests", func() {
 				By("Cleaning up target DV")
 				err = utils.DeleteDataVolume(f.CdiClient, f.Namespace.Name, targetDv.Name)
 				Expect(err).ToNot(HaveOccurred())
+				if targetDv.Status.Phase == cdiv1.Succeeded {
+					validateCloneType(f, targetDv)
+				}
 			}
 		})
 
@@ -485,6 +578,9 @@ var _ = Describe("all clone tests", func() {
 			dvs := []*cdiv1.DataVolume{sourceDv, targetDv1, targetDv2, targetDv3}
 			for _, dv := range dvs {
 				cleanDv(f, dv)
+				if dv != nil && dv.Status.Phase == cdiv1.Succeeded {
+					validateCloneType(f, dv)
+				}
 			}
 		})
 
@@ -604,13 +700,15 @@ var _ = Describe("all clone tests", func() {
 	var _ = Describe("Block PV Cloner Test", func() {
 		f := framework.NewFramework(namespacePrefix)
 
-		It("[test_id:4955]Should clone data across namespaces", func() {
+		DescribeTable("[test_id:4955]Should clone data across namespaces", func(targetSize string) {
 			if !f.IsBlockVolumeStorageClassAvailable() {
 				Skip("Storage Class for block volume is not available")
 			}
-			pvcDef := utils.NewBlockPVCDefinition(sourcePVCName, "500M", nil, nil, f.BlockSCName)
+			sourceSize := "500M"
+			ss := resource.MustParse(sourceSize)
+			pvcDef := utils.NewBlockPVCDefinition(sourcePVCName, sourceSize, nil, nil, f.BlockSCName)
 			sourcePvc := f.CreateAndPopulateSourcePVC(pvcDef, "fill-source-block-pod", blockFillCommand)
-			sourceMD5, err := f.GetMD5(f.Namespace, sourcePvc, testBaseDir, 0)
+			sourceMD5, err := f.GetMD5(f.Namespace, sourcePvc, testBaseDir, ss.Value())
 			Expect(err).ToNot(HaveOccurred())
 
 			By("Deleting verifier pod")
@@ -623,7 +721,7 @@ var _ = Describe("all clone tests", func() {
 			Expect(err).NotTo(HaveOccurred())
 			f.AddNamespaceToDelete(targetNs)
 
-			targetDV := utils.NewDataVolumeCloneToBlockPV("target-dv", "500M", sourcePvc.Namespace, sourcePvc.Name, f.BlockSCName)
+			targetDV := utils.NewDataVolumeCloneToBlockPV("target-dv", targetSize, sourcePvc.Namespace, sourcePvc.Name, f.BlockSCName)
 			dataVolume, err := utils.CreateDataVolumeFromDefinition(f.CdiClient, targetNs.Name, targetDV)
 			Expect(err).ToNot(HaveOccurred())
 
@@ -635,11 +733,22 @@ var _ = Describe("all clone tests", func() {
 
 			err = utils.WaitForDataVolumePhaseWithTimeout(f.CdiClient, targetNs.Name, cdiv1.Succeeded, "target-dv", 3*90*time.Second)
 			Expect(err).ToNot(HaveOccurred())
-			Expect(f.VerifyTargetPVCContentMD5(targetNs, targetPvc, testBaseDir, sourceMD5)).To(BeTrue())
+			Expect(f.VerifyTargetPVCContentMD5(targetNs, targetPvc, testBaseDir, sourceMD5, ss.Value())).To(BeTrue())
 			By("Deleting verifier pod")
 			err = utils.DeleteVerifierPod(f.K8sClient, targetNs.Name)
 			Expect(err).ToNot(HaveOccurred())
-		})
+
+			validateCloneType(f, dataVolume)
+
+			targetPvc, err = f.K8sClient.CoreV1().PersistentVolumeClaims(targetPvc.Namespace).Get(context.TODO(), targetPvc.Name, metav1.GetOptions{})
+			Expect(err).ToNot(HaveOccurred())
+
+			es := resource.MustParse(targetSize)
+			Expect(es.Cmp(*targetPvc.Status.Capacity.Storage()) <= 0).To(BeTrue())
+		},
+			Entry("with same target size", "500M"),
+			Entry("with bigger target", "1G"),
+		)
 	})
 
 	var _ = Describe("Namespace with quota", func() {
@@ -703,6 +812,7 @@ var _ = Describe("all clone tests", func() {
 			err := f.UpdateCdiConfigResourceLimits(int64(2), int64(1024*1024*1024), int64(2), int64(1*1024*1024*1024))
 			Expect(err).NotTo(HaveOccurred())
 			err = f.CreateQuotaInNs(int64(1), int64(1024*1024*1024), int64(2), int64(2*1024*1024*1024))
+			Expect(err).NotTo(HaveOccurred())
 			smartApplicable := f.IsSnapshotStorageClassAvailable()
 			sc, err := f.K8sClient.StorageV1().StorageClasses().Get(context.TODO(), f.SnapshotSCName, metav1.GetOptions{})
 			if err == nil {
@@ -724,7 +834,7 @@ var _ = Describe("all clone tests", func() {
 			f.ForceBindPvcIfDvIsWaitForFirstConsumer(dataVolume)
 
 			By("Verify Quota was exceeded in logs")
-			matchString := fmt.Sprintf("\\\"cdi-upload-target-dv\\\" is forbidden: exceeded quota: test-quota, requested")
+			matchString := "\\\"cdi-upload-target-dv\\\" is forbidden: exceeded quota: test-quota, requested"
 			Eventually(func() string {
 				log, err := RunKubectlCommand(f, "logs", f.ControllerPod.Name, "-n", f.CdiInstallNs)
 				Expect(err).NotTo(HaveOccurred())
@@ -758,7 +868,7 @@ var _ = Describe("all clone tests", func() {
 			f.ForceBindPvcIfDvIsWaitForFirstConsumer(dataVolume)
 
 			By("Verify Quota was exceeded in logs")
-			matchString := fmt.Sprintf("\\\"cdi-upload-target-dv\\\" is forbidden: exceeded quota: test-quota, requested")
+			matchString := "\\\"cdi-upload-target-dv\\\" is forbidden: exceeded quota: test-quota, requested"
 			Eventually(func() string {
 				log, err := RunKubectlCommand(f, "logs", f.ControllerPod.Name, "-n", f.CdiInstallNs)
 				Expect(err).NotTo(HaveOccurred())
@@ -811,6 +921,12 @@ var _ = Describe("all clone tests", func() {
 			targetDV := utils.NewDataVolumeForImageCloning("target-dv", "500M", sourcePvc.Namespace, sourcePvc.Name, sourcePvc.Spec.StorageClassName, sourcePvc.Spec.VolumeMode)
 			dataVolume, err := utils.CreateDataVolumeFromDefinition(f.CdiClient, targetNs.Name, targetDV)
 			Expect(err).ToNot(HaveOccurred())
+
+			cloneType := utils.GetCloneType(f.CdiClient, dataVolume)
+			if cloneType != "network" {
+				Skip("only valid for network clone")
+			}
+
 			f.ForceBindPvcIfDvIsWaitForFirstConsumer(dataVolume)
 
 			By("Verify Quota was exceeded in logs")
@@ -841,6 +957,12 @@ var _ = Describe("all clone tests", func() {
 			targetDV := utils.NewDataVolumeForImageCloning("target-dv", "500M", sourcePvc.Namespace, sourcePvc.Name, sourcePvc.Spec.StorageClassName, sourcePvc.Spec.VolumeMode)
 			dataVolume, err := utils.CreateDataVolumeFromDefinition(f.CdiClient, targetNs.Name, targetDV)
 			Expect(err).ToNot(HaveOccurred())
+
+			cloneType := utils.GetCloneType(f.CdiClient, dataVolume)
+			if cloneType != "network" {
+				Skip("only valid for network clone")
+			}
+
 			f.ForceBindPvcIfDvIsWaitForFirstConsumer(dataVolume)
 
 			By("Verify Quota was exceeded in logs")
@@ -885,6 +1007,14 @@ var _ = Describe("all clone tests", func() {
 			targetDvName := "target-dv"
 			doFileBasedCloneTest(f, pvcDef, targetNs, targetDvName)
 
+			dv, err := f.CdiClient.CdiV1beta1().DataVolumes(targetNs.Name).Get(context.TODO(), targetDvName, metav1.GetOptions{})
+			Expect(err).NotTo(HaveOccurred())
+
+			cloneType := utils.GetCloneType(f.CdiClient, dv)
+			if cloneType != "network" {
+				Skip("only valid for network clone")
+			}
+
 			By("Verify retry annotation on PVC")
 			targetPvc, err := utils.WaitForPVC(f.K8sClient, targetNs.Name, targetDvName)
 			Expect(err).ToNot(HaveOccurred())
@@ -894,8 +1024,6 @@ var _ = Describe("all clone tests", func() {
 			Expect(restartsValue).To(Equal("0"))
 
 			By("Verify the number of retries on the datavolume")
-			dv, err := f.CdiClient.CdiV1beta1().DataVolumes(targetNs.Name).Get(context.TODO(), targetDvName, metav1.GetOptions{})
-			Expect(err).NotTo(HaveOccurred())
 			Expect(dv.Status.RestartCount).To(BeNumerically("==", 0))
 		})
 
@@ -915,6 +1043,11 @@ var _ = Describe("all clone tests", func() {
 			dataVolume, err := utils.CreateDataVolumeFromDefinition(f.CdiClient, targetNs.Name, targetDV)
 			Expect(err).ToNot(HaveOccurred())
 
+			cloneType := utils.GetCloneType(f.CdiClient, dataVolume)
+			if cloneType != "network" {
+				Skip("only valid for network clone")
+			}
+
 			targetPvc, err := utils.WaitForPVC(f.K8sClient, dataVolume.Namespace, dataVolume.Name)
 			Expect(err).ToNot(HaveOccurred())
 			f.ForceBindIfWaitForFirstConsumer(targetPvc)
@@ -969,18 +1102,24 @@ var _ = Describe("all clone tests", func() {
 
 			doFileBasedCloneTest(f, pvcDef, targetNs, targetDvName)
 
-			By("Verify retry annotation on PVC")
-			targetPvc, err := utils.WaitForPVC(f.K8sClient, targetNs.Name, targetDvName)
-			Expect(err).ToNot(HaveOccurred())
-			restartsValue, status, err := utils.WaitForPVCAnnotation(f.K8sClient, targetNs.Name, targetPvc, controller.AnnPodRestarts)
-			Expect(err).ToNot(HaveOccurred())
-			Expect(status).To(BeTrue())
-			Expect(restartsValue).To(Equal("0"))
-
-			By("Verify the number of retries on the datavolume")
 			dv, err := f.CdiClient.CdiV1beta1().DataVolumes(targetNs.Name).Get(context.TODO(), targetDvName, metav1.GetOptions{})
 			Expect(err).NotTo(HaveOccurred())
-			Expect(dv.Status.RestartCount).To(BeNumerically("==", 0))
+
+			cloneType := utils.GetCloneType(f.CdiClient, dv)
+			if cloneType == "network" {
+				By("Verify retry annotation on PVC")
+				targetPvc, err := utils.WaitForPVC(f.K8sClient, targetNs.Name, targetDvName)
+				Expect(err).ToNot(HaveOccurred())
+				restartsValue, status, err := utils.WaitForPVCAnnotation(f.K8sClient, targetNs.Name, targetPvc, controller.AnnPodRestarts)
+				Expect(err).ToNot(HaveOccurred())
+				Expect(status).To(BeTrue())
+				Expect(restartsValue).To(Equal("0"))
+
+				By("Verify the number of retries on the datavolume")
+				dv, err := f.CdiClient.CdiV1beta1().DataVolumes(targetNs.Name).Get(context.TODO(), targetDvName, metav1.GetOptions{})
+				Expect(err).NotTo(HaveOccurred())
+				Expect(dv.Status.RestartCount).To(BeNumerically("==", 0))
+			}
 		})
 
 		It("[test_id:4277] Clone datavolume with long name", func() {
@@ -1002,18 +1141,24 @@ var _ = Describe("all clone tests", func() {
 
 			doFileBasedCloneTest(f, pvcDef, targetNs, targetDvName)
 
-			By("Verify retry annotation on PVC")
-			targetPvc, err := utils.WaitForPVC(f.K8sClient, targetNs.Name, targetDvName)
-			Expect(err).ToNot(HaveOccurred())
-			restartsValue, status, err := utils.WaitForPVCAnnotation(f.K8sClient, targetNs.Name, targetPvc, controller.AnnPodRestarts)
-			Expect(err).ToNot(HaveOccurred())
-			Expect(status).To(BeTrue())
-			Expect(restartsValue).To(Equal("0"))
-
-			By("Verify the number of retries on the datavolume")
 			dv, err := f.CdiClient.CdiV1beta1().DataVolumes(targetNs.Name).Get(context.TODO(), targetDvName, metav1.GetOptions{})
 			Expect(err).NotTo(HaveOccurred())
-			Expect(dv.Status.RestartCount).To(BeNumerically("==", 0))
+
+			cloneType := utils.GetCloneType(f.CdiClient, dv)
+			if cloneType == "network" {
+				By("Verify retry annotation on PVC")
+				targetPvc, err := utils.WaitForPVC(f.K8sClient, targetNs.Name, targetDvName)
+				Expect(err).ToNot(HaveOccurred())
+				restartsValue, status, err := utils.WaitForPVCAnnotation(f.K8sClient, targetNs.Name, targetPvc, controller.AnnPodRestarts)
+				Expect(err).ToNot(HaveOccurred())
+				Expect(status).To(BeTrue())
+				Expect(restartsValue).To(Equal("0"))
+
+				By("Verify the number of retries on the datavolume")
+				dv, err := f.CdiClient.CdiV1beta1().DataVolumes(targetNs.Name).Get(context.TODO(), targetDvName, metav1.GetOptions{})
+				Expect(err).NotTo(HaveOccurred())
+				Expect(dv.Status.RestartCount).To(BeNumerically("==", 0))
+			}
 		})
 
 		It("[test_id:4278] Clone datavolume with long name including special character '.'", func() {
@@ -1035,26 +1180,128 @@ var _ = Describe("all clone tests", func() {
 
 			doFileBasedCloneTest(f, pvcDef, targetNs, targetDvName)
 
-			By("Verify retry annotation on PVC")
-			targetPvc, err := utils.WaitForPVC(f.K8sClient, targetNs.Name, targetDvName)
-			Expect(err).ToNot(HaveOccurred())
-			restartsValue, status, err := utils.WaitForPVCAnnotation(f.K8sClient, targetNs.Name, targetPvc, controller.AnnPodRestarts)
-			Expect(err).ToNot(HaveOccurred())
-			Expect(status).To(BeTrue())
-			Expect(restartsValue).To(Equal("0"))
-
-			By("Verify the number of retries on the datavolume")
 			dv, err := f.CdiClient.CdiV1beta1().DataVolumes(targetNs.Name).Get(context.TODO(), targetDvName, metav1.GetOptions{})
 			Expect(err).NotTo(HaveOccurred())
-			Expect(dv.Status.RestartCount).To(BeNumerically("==", 0))
+
+			cloneType := utils.GetCloneType(f.CdiClient, dv)
+			if cloneType == "network" {
+				By("Verify retry annotation on PVC")
+				targetPvc, err := utils.WaitForPVC(f.K8sClient, targetNs.Name, targetDvName)
+				Expect(err).ToNot(HaveOccurred())
+				restartsValue, status, err := utils.WaitForPVCAnnotation(f.K8sClient, targetNs.Name, targetPvc, controller.AnnPodRestarts)
+				Expect(err).ToNot(HaveOccurred())
+				Expect(status).To(BeTrue())
+				Expect(restartsValue).To(Equal("0"))
+
+				By("Verify the number of retries on the datavolume")
+				dv, err := f.CdiClient.CdiV1beta1().DataVolumes(targetNs.Name).Get(context.TODO(), targetDvName, metav1.GetOptions{})
+				Expect(err).NotTo(HaveOccurred())
+				Expect(dv.Status.RestartCount).To(BeNumerically("==", 0))
+			}
+		})
+	})
+
+	var _ = Describe("Preallocation", func() {
+		f := framework.NewFramework(namespacePrefix)
+
+		var sourcePvc *v1.PersistentVolumeClaim
+		var targetPvc *v1.PersistentVolumeClaim
+		var cdiCr cdiv1.CDI
+		var cdiCrSpec *cdiv1.CDISpec
+
+		BeforeEach(func() {
+			By("[BeforeEach] Saving CDI CR spec")
+			crList, err := f.CdiClient.CdiV1beta1().CDIs().List(context.TODO(), metav1.ListOptions{})
+			Expect(err).ToNot(HaveOccurred())
+			Expect(len(crList.Items)).To(Equal(1))
+
+			cdiCrSpec = crList.Items[0].Spec.DeepCopy()
+			cdiCr = crList.Items[0]
+
+			By("[BeforeEach] Forcing Host Assisted cloning")
+			var cloneStrategy cdiv1.CDICloneStrategy = cdiv1.CloneStrategyHostAssisted
+			cdiCr.Spec.CloneStrategyOverride = &cloneStrategy
+			_, err = f.CdiClient.CdiV1beta1().CDIs().Update(context.TODO(), &cdiCr, metav1.UpdateOptions{})
+			Expect(err).ToNot(HaveOccurred())
+
+			err = utils.WaitForCDICrCloneStrategy(f.CdiClient, cloneStrategy)
+			Expect(err).ToNot(HaveOccurred())
 		})
 
+		AfterEach(func() {
+			if sourcePvc != nil {
+				By("[AfterEach] Clean up source PVC")
+				err := f.DeletePVC(sourcePvc)
+				Expect(err).ToNot(HaveOccurred())
+			}
+			if targetPvc != nil {
+				By("[AfterEach] Clean up target PVC")
+				err := f.DeletePVC(targetPvc)
+				Expect(err).ToNot(HaveOccurred())
+			}
+
+			By("[AfterEach] Restoring CDI CR spec to original state")
+			crList, err := f.CdiClient.CdiV1beta1().CDIs().List(context.TODO(), metav1.ListOptions{})
+			Expect(err).ToNot(HaveOccurred())
+			Expect(len(crList.Items)).To(Equal(1))
+
+			newCdiCr := crList.Items[0]
+			newCdiCr.Spec = *cdiCrSpec
+			_, err = f.CdiClient.CdiV1beta1().CDIs().Update(context.TODO(), &newCdiCr, metav1.UpdateOptions{})
+			Expect(err).ToNot(HaveOccurred())
+
+			if cdiCrSpec.CloneStrategyOverride == nil {
+				err = utils.WaitForCDICrCloneStrategyNil(f.CdiClient)
+			} else {
+				err = utils.WaitForCDICrCloneStrategy(f.CdiClient, *cdiCrSpec.CloneStrategyOverride)
+			}
+			Expect(err).ToNot(HaveOccurred())
+		})
+
+		It("Should preallocate data on target PVC", func() {
+			dataVolume := utils.NewDataVolumeWithHTTPImport(dataVolumeName, "1Gi", fmt.Sprintf(utils.TinyCoreIsoURL, f.CdiInstallNs))
+			dataVolume, err := utils.CreateDataVolumeFromDefinition(f.CdiClient, f.Namespace.Name, dataVolume)
+			Expect(err).ToNot(HaveOccurred())
+
+			f.ForceBindPvcIfDvIsWaitForFirstConsumer(dataVolume)
+			sourcePvc, err = f.K8sClient.CoreV1().PersistentVolumeClaims(dataVolume.Namespace).Get(context.TODO(), dataVolume.Name, metav1.GetOptions{})
+
+			targetDV := utils.NewCloningDataVolume("target-dv", "1Gi", sourcePvc)
+			preallocation := true
+			targetDV.Spec.Preallocation = &preallocation
+			targetDataVolume, err := utils.CreateDataVolumeFromDefinition(f.CdiClient, f.Namespace.Name, targetDV)
+			Expect(err).ToNot(HaveOccurred())
+			f.ForceBindPvcIfDvIsWaitForFirstConsumer(targetDataVolume)
+
+			targetPvc, err = utils.WaitForPVC(f.K8sClient, targetDataVolume.Namespace, targetDataVolume.Name)
+			Expect(err).ToNot(HaveOccurred())
+			annValue, preallocationAnnotationFound, err := utils.WaitForPVCAnnotation(f.K8sClient, targetDataVolume.Namespace, targetPvc, controller.AnnPreallocationRequested)
+			if err != nil {
+				PrintControllerLog(f)
+			}
+			Expect(err).ToNot(HaveOccurred())
+			Expect(preallocationAnnotationFound).To(BeTrue())
+			Expect(annValue).To(Equal("true"))
+
+			fmt.Fprintf(GinkgoWriter, "INFO: wait for target DV phase Succeeded: %s\n", targetPvc.Name)
+
+			annValue, preallocationAnnotationFound, err = utils.WaitForPVCAnnotation(f.K8sClient, targetDataVolume.Namespace, targetPvc, controller.AnnPreallocationApplied)
+			if err != nil {
+				PrintControllerLog(f)
+			}
+			Expect(err).ToNot(HaveOccurred())
+			Expect(preallocationAnnotationFound).To(BeTrue())
+			Expect(annValue).To(Equal("true"))
+		})
 	})
 })
 
-func doFileBasedCloneTest(f *framework.Framework, srcPVCDef *v1.PersistentVolumeClaim, targetNs *v1.Namespace, targetDv string) {
+func doFileBasedCloneTest(f *framework.Framework, srcPVCDef *v1.PersistentVolumeClaim, targetNs *v1.Namespace, targetDv string, targetSize ...string) {
+	if len(targetSize) == 0 {
+		targetSize = []string{"1G"}
+	}
 	// Create targetPvc in new NS.
-	targetDV := utils.NewCloningDataVolume(targetDv, "1G", srcPVCDef)
+	targetDV := utils.NewCloningDataVolume(targetDv, targetSize[0], srcPVCDef)
 	dataVolume, err := utils.CreateDataVolumeFromDefinition(f.CdiClient, targetNs.Name, targetDV)
 	Expect(err).ToNot(HaveOccurred())
 
@@ -1069,6 +1316,12 @@ func doFileBasedCloneTest(f *framework.Framework, srcPVCDef *v1.PersistentVolume
 	Expect(err).ToNot(HaveOccurred())
 
 	completeClone(f, targetNs, targetPvc, filepath.Join(testBaseDir, testFile), fillDataFSMD5sum, sourcePvcDiskGroup)
+
+	targetPvc, err = f.K8sClient.CoreV1().PersistentVolumeClaims(targetPvc.Namespace).Get(context.TODO(), targetPvc.Name, metav1.GetOptions{})
+	Expect(err).ToNot(HaveOccurred())
+
+	es := resource.MustParse(targetSize[0])
+	Expect(es.Cmp(*targetPvc.Status.Capacity.Storage()) <= 0).To(BeTrue())
 }
 
 func doInUseCloneTest(f *framework.Framework, srcPVCDef *v1.PersistentVolumeClaim, targetNs *v1.Namespace, targetDv string) {
@@ -1086,13 +1339,26 @@ func doInUseCloneTest(f *framework.Framework, srcPVCDef *v1.PersistentVolumeClai
 	dataVolume, err := utils.CreateDataVolumeFromDefinition(f.CdiClient, targetNs.Name, targetDV)
 	Expect(err).ToNot(HaveOccurred())
 
-	targetPvc, err := utils.WaitForPVC(f.K8sClient, dataVolume.Namespace, dataVolume.Name)
-	Expect(err).ToNot(HaveOccurred())
-	f.ForceBindPvcIfDvIsWaitForFirstConsumer(dataVolume)
+	var targetPvc *v1.PersistentVolumeClaim
+	cloneType := utils.GetCloneType(f.CdiClient, dataVolume)
 
-	verifyEvent(controller.CloneSourceInUse, targetNs.Name, f)
-	err = f.K8sClient.CoreV1().Pods(f.Namespace.Name).Delete(context.TODO(), pod.Name, metav1.DeleteOptions{})
-	Expect(err).ToNot(HaveOccurred())
+	if cloneType == "network" {
+		targetPvc, err = utils.WaitForPVC(f.K8sClient, dataVolume.Namespace, dataVolume.Name)
+		Expect(err).ToNot(HaveOccurred())
+		f.ForceBindPvcIfDvIsWaitForFirstConsumer(dataVolume)
+
+		verifyEvent(controller.CloneSourceInUse, targetNs.Name, f)
+		err = f.K8sClient.CoreV1().Pods(f.Namespace.Name).Delete(context.TODO(), pod.Name, metav1.DeleteOptions{})
+		Expect(err).ToNot(HaveOccurred())
+	} else {
+		verifyEvent(controller.SmartCloneSourceInUse, targetNs.Name, f)
+		err = f.K8sClient.CoreV1().Pods(f.Namespace.Name).Delete(context.TODO(), pod.Name, metav1.DeleteOptions{})
+		Expect(err).ToNot(HaveOccurred())
+
+		targetPvc, err = utils.WaitForPVC(f.K8sClient, dataVolume.Namespace, dataVolume.Name)
+		Expect(err).ToNot(HaveOccurred())
+		f.ForceBindPvcIfDvIsWaitForFirstConsumer(dataVolume)
+	}
 
 	fmt.Fprintf(GinkgoWriter, "INFO: wait for PVC claim phase: %s\n", targetPvc.Name)
 	utils.WaitForPersistentVolumeClaimPhase(f.K8sClient, targetNs.Name, v1.ClaimBound, targetPvc.Name)
@@ -1114,6 +1380,7 @@ func completeClone(f *framework.Framework, targetNs *v1.Namespace, targetPvc *v1
 
 	By("Verify the clone status is success on the target datavolume")
 	err = utils.WaitForDataVolumePhase(f.CdiClient, targetNs.Name, cdiv1.Succeeded, targetPvc.Name)
+	Expect(err).ToNot(HaveOccurred())
 
 	By("Verify the content")
 	Expect(f.VerifyTargetPVCContentMD5(targetNs, targetPvc, filePath, expectedMD5)).To(BeTrue())
@@ -1128,6 +1395,25 @@ func completeClone(f *framework.Framework, targetNs *v1.Namespace, targetPvc *v1
 	By("Deleting verifier pod")
 	err = utils.DeleteVerifierPod(f.K8sClient, targetNs.Name)
 	Expect(err).ToNot(HaveOccurred())
+
+	dv, err := f.CdiClient.CdiV1beta1().DataVolumes(targetNs.Name).Get(context.TODO(), targetPvc.Name, metav1.GetOptions{})
+	Expect(err).ToNot(HaveOccurred())
+
+	validateCloneType(f, dv)
+
+	if utils.GetCloneType(f.CdiClient, dv) == "snapshot" {
+		sns := dv.Spec.Source.PVC.Namespace
+		if sns == "" {
+			sns = dv.Namespace
+		}
+
+		snapshots := &snapshotv1.VolumeSnapshotList{}
+		err = f.CrClient.List(context.TODO(), snapshots, &client.ListOptions{Namespace: sns})
+		Expect(err).ToNot(HaveOccurred())
+		for _, s := range snapshots.Items {
+			Expect(s.DeletionTimestamp).ToNot(BeNil())
+		}
+	}
 }
 
 func cloneOfAnnoExistenceTest(f *framework.Framework, targetNamespaceName string) {
@@ -1170,3 +1456,38 @@ func cleanDv(f *framework.Framework, dv *cdiv1.DataVolume) {
 		Expect(err).ToNot(HaveOccurred())
 	}
 }
+
+func validateCloneType(f *framework.Framework, dv *cdiv1.DataVolume) {
+	if dv.Spec.Source.PVC == nil {
+		return
+	}
+
+	cloneType := "network"
+
+	if f.IsSnapshotStorageClassAvailable() {
+		sourceNamespace := dv.Namespace
+		if dv.Spec.Source.PVC.Namespace != "" {
+			sourceNamespace = dv.Spec.Source.PVC.Namespace
+		}
+
+		sc, err := f.K8sClient.StorageV1().StorageClasses().Get(context.TODO(), f.SnapshotSCName, metav1.GetOptions{})
+		Expect(err).ToNot(HaveOccurred())
+		allowsExpansion := sc.AllowVolumeExpansion != nil && *sc.AllowVolumeExpansion
+
+		sourcePVC, err := f.K8sClient.CoreV1().PersistentVolumeClaims(sourceNamespace).Get(context.TODO(), dv.Spec.Source.PVC.Name, metav1.GetOptions{})
+		Expect(err).ToNot(HaveOccurred())
+
+		targetPVC, err := f.K8sClient.CoreV1().PersistentVolumeClaims(dv.Namespace).Get(context.TODO(), dv.Name, metav1.GetOptions{})
+		Expect(err).ToNot(HaveOccurred())
+
+		if sourcePVC.Spec.StorageClassName != nil &&
+			targetPVC.Spec.StorageClassName != nil &&
+			*sourcePVC.Spec.StorageClassName == *targetPVC.Spec.StorageClassName &&
+			*sourcePVC.Spec.StorageClassName == f.SnapshotSCName &&
+			(allowsExpansion || sourcePVC.Status.Capacity.Storage().Cmp(*targetPVC.Status.Capacity.Storage()) == 0) {
+			cloneType = "snapshot"
+		}
+	}
+
+	Expect(utils.GetCloneType(f.CdiClient, dv)).To(Equal(cloneType))
+}
