@@ -13,6 +13,7 @@ import (
 	"strings"
 	"time"
 
+	"github.com/google/uuid"
 	. "github.com/onsi/ginkgo"
 	. "github.com/onsi/ginkgo/extensions/table"
 	. "github.com/onsi/gomega"
@@ -74,7 +75,7 @@ var _ = Describe("[rfe_id:1115][crit:high][vendor:cnv-qe@redhat.com][level:compo
 			if strings.Contains(log, "no space left on device") {
 				return true
 			}
-			if strings.Contains(log, "qemu-img execution failed") {
+			if strings.Contains(log, "file largest block is bigger than maxblock") {
 				return true
 			}
 			By("Failed to find error messages about a too large image in log:")
@@ -142,8 +143,8 @@ var _ = Describe("[rfe_id:4784][crit:high] Importer respects node placement", fu
 	f := framework.NewFramework(namespacePrefix)
 
 	// An image that fails import
-	invalidQcowLargeSize := func() string {
-		return fmt.Sprintf(utils.InvalidQcowImagesURL+"invalid-qcow-large-size.img", f.CdiInstallNs)
+	imageTooLargeSize := func() string {
+		return fmt.Sprintf(utils.LargeVirtualDiskXz, f.CdiInstallNs)
 	}
 
 	BeforeEach(func() {
@@ -186,7 +187,7 @@ var _ = Describe("[rfe_id:4784][crit:high] Importer respects node placement", fu
 			return reflect.DeepEqual(cr.Spec, realCR.Spec)
 		}, 30*time.Second, time.Second)
 
-		dv := utils.NewDataVolumeWithHTTPImport("node-placement-test", "100Mi", invalidQcowLargeSize())
+		dv := utils.NewDataVolumeWithHTTPImport("node-placement-test", "100Mi", imageTooLargeSize())
 		dv, err = utils.CreateDataVolumeFromDefinition(f.CdiClient, f.Namespace.Name, dv)
 		Expect(err).ToNot(HaveOccurred())
 
@@ -204,11 +205,15 @@ var _ = Describe("[rfe_id:4784][crit:high] Importer respects node placement", fu
 })
 
 var _ = Describe("Importer CDI config manipulation tests", func() {
-	var config *cdiv1.CDIConfig
-	var origSpec *cdiv1.CDIConfigSpec
-	var err error
 	f := framework.NewFramework(namespacePrefix)
 
+	var (
+		config         *cdiv1.CDIConfig
+		origSpec       *cdiv1.CDIConfigSpec
+		err            error
+		tinyCoreIsoURL = func() string { return fmt.Sprintf(utils.TinyCoreIsoURL, f.CdiInstallNs) }
+	)
+
 	BeforeEach(func() {
 		config, err = f.CdiClient.CdiV1beta1().CDIConfigs().Get(context.TODO(), common.ConfigName, metav1.GetOptions{})
 		Expect(err).ToNot(HaveOccurred())
@@ -229,35 +234,21 @@ var _ = Describe("Importer CDI config manipulation tests", func() {
 		}, 30*time.Second, time.Second)
 	})
 
-	DescribeTable("Filesystem overhead is honored with a RAW file", func(expectedSuccess bool, globalOverhead, scOverhead string) {
-		defaultSCName := utils.DefaultStorageClass.GetName()
-		testedFilesystemOverhead := &cdiv1.FilesystemOverhead{}
-		if globalOverhead != "" {
-			testedFilesystemOverhead.Global = cdiv1.Percent(globalOverhead)
-		}
-		if scOverhead != "" {
-			testedFilesystemOverhead.StorageClass = map[string]cdiv1.Percent{defaultSCName: cdiv1.Percent(scOverhead)}
-		}
-		config.Spec.FilesystemOverhead = testedFilesystemOverhead.DeepCopy()
-		By(fmt.Sprintf("Updating CDIConfig filesystem overhead to %v", config.Spec.FilesystemOverhead))
-		err := utils.UpdateCDIConfig(f.CrClient, func(config *cdiv1.CDIConfigSpec) {
-			config.FilesystemOverhead = testedFilesystemOverhead.DeepCopy()
-		})
-		Expect(err).ToNot(HaveOccurred())
-		By(fmt.Sprintf("Waiting for filsystem overhead status to be set to %v", testedFilesystemOverhead))
-		Eventually(func() bool {
-			config, err := f.CdiClient.CdiV1beta1().CDIConfigs().Get(context.TODO(), common.ConfigName, metav1.GetOptions{})
-			Expect(err).ToNot(HaveOccurred())
-			if scOverhead != "" {
-				return config.Status.FilesystemOverhead.StorageClass[defaultSCName] == cdiv1.Percent(scOverhead)
-			}
-			return config.Status.FilesystemOverhead.StorageClass[defaultSCName] == cdiv1.Percent(globalOverhead)
-		}, timeout, pollingInterval).Should(BeTrue(), "CDIConfig filesystem overhead wasn't set")
+	dvWithPvcSpec := func() *cdiv1.DataVolume {
+		return utils.NewDataVolumeWithHTTPImport("too-large-import", "50Mi", tinyCoreIsoURL())
+	}
+	dvWithStorageSpec := func() *cdiv1.DataVolume {
+		return utils.NewDataVolumeWithHTTPImportAndStorageSpec("too-large-import", "50Mi", tinyCoreIsoURL())
+	}
+	// value of .80 means 80% of filesystem is reserved for metadata,
+	// given 100Mi PVC, 80Mi is reserved, 20Mi available for image, we test using 50Mi PVC and around 20Mi image
+	highFsOverhead := "0.80"
 
-		imageURL := fmt.Sprintf(utils.TinyCoreIsoURL, f.CdiInstallNs)
+	DescribeTable("Filesystem overhead is honored with a RAW file", func(dvFunc func() *cdiv1.DataVolume, expectedSuccess bool, globalOverhead, scOverhead string) {
+		tests.SetFilesystemOverhead(f, globalOverhead, scOverhead)
 
-		By(imageURL)
-		dv := utils.NewDataVolumeWithHTTPImport("too-large-import", "500Mi", imageURL)
+		dv := dvFunc()
+		By(dv.Name)
 		dv, err = utils.CreateDataVolumeFromDefinition(f.CdiClient, f.Namespace.Name, dv)
 		Expect(err).ToNot(HaveOccurred())
 
@@ -283,19 +274,17 @@ var _ = Describe("Importer CDI config manipulation tests", func() {
 				if strings.Contains(log, "no space left on device") {
 					return true
 				}
-				if strings.Contains(log, "qemu-img execution failed") {
-					return true
-				}
 				By("Failed to find error messages about a too large image in log:")
 				By(log)
 				return false
 			}, controllerSkipPVCCompleteTimeout, assertionPollInterval).Should(BeTrue())
 		}
 	},
-		Entry("Succeed with low global overhead", true, "0.1", ""),
-		Entry("[posneg:negative] Fail with high global overhead", false, "0.99", ""),
-		Entry("Succeed with low per-storageclass overhead (despite high global overhead)", true, "0.99", "0.1"),
-		Entry("[posneg:negative] Fail with high per-storageclass overhead (despite low global overhead)", false, "0.1", "0.99"),
+		Entry("Succeed with low global overhead", dvWithPvcSpec, true, "0.1", ""),
+		Entry("[posneg:negative] Fail with high global overhead", dvWithPvcSpec, false, highFsOverhead, ""),
+		Entry("Succeed with high global overhead when using spec.storage", dvWithStorageSpec, true, highFsOverhead, ""),
+		Entry("Succeed with low per-storageclass overhead (despite high global overhead)", dvWithPvcSpec, true, highFsOverhead, "0.1"),
+		Entry("[posneg:negative] Fail with high per-storageclass overhead (despite low global overhead)", dvWithPvcSpec, false, "0.1", highFsOverhead),
 	)
 })
 
@@ -938,9 +927,19 @@ var _ = Describe("Preallocation", func() {
 		tinyCoreTarURL      = func() string { return fmt.Sprintf(utils.TarArchiveURL, f.CdiInstallNs) }
 		tinyCoreRegistryURL = func() string { return fmt.Sprintf(utils.TinyCoreIsoRegistryURL, f.CdiInstallNs) }
 		imageioURL          = func() string { return fmt.Sprintf(utils.ImageioURL, f.CdiInstallNs) }
+		vcenterURL          = func() string { return fmt.Sprintf(utils.VcenterURL, f.CdiInstallNs) }
 		md5PrefixSize       = int64(100000)
+		config              *cdiv1.CDIConfig
+		origSpec            *cdiv1.CDIConfigSpec
 	)
 
+	BeforeEach(func() {
+		config, err = f.CdiClient.CdiV1beta1().CDIConfigs().Get(context.TODO(), common.ConfigName, metav1.GetOptions{})
+		Expect(err).ToNot(HaveOccurred())
+
+		origSpec = config.Spec.DeepCopy()
+	})
+
 	AfterEach(func() {
 		By("Delete DV")
 		err := utils.DeleteDataVolume(f.CdiClient, f.Namespace.Name, dataVolume.Name)
@@ -953,6 +952,17 @@ var _ = Describe("Preallocation", func() {
 			}
 			return false
 		}, timeout, pollingInterval).Should(BeTrue())
+
+		By("Restoring CDIConfig to original state")
+		err = utils.UpdateCDIConfig(f.CrClient, func(config *cdiv1.CDIConfigSpec) {
+			origSpec.DeepCopyInto(config)
+		})
+
+		Eventually(func() bool {
+			config, err = f.CdiClient.CdiV1beta1().CDIConfigs().Get(context.TODO(), "cdi", metav1.GetOptions{})
+			Expect(err).ToNot(HaveOccurred())
+			return reflect.DeepEqual(config.Spec, origSpec)
+		}, 30*time.Second, time.Second)
 	})
 
 	It("Importer should add preallocation when requested", func() {
@@ -980,6 +990,10 @@ var _ = Describe("Preallocation", func() {
 		md5, err := f.GetMD5(f.Namespace, pvc, "/pvc/disk.img", md5PrefixSize)
 		Expect(err).ToNot(HaveOccurred())
 		Expect(md5).To(Equal(utils.TinyCoreMD5))
+
+		ok, err := f.VerifyImagePreallocated(f.Namespace, pvc)
+		Expect(err).ToNot(HaveOccurred())
+		Expect(ok).To(BeTrue())
 	})
 
 	It("Importer should not add preallocation when preallocation=false", func() {
@@ -1008,6 +1022,11 @@ var _ = Describe("Preallocation", func() {
 		md5, err := f.GetMD5(f.Namespace, pvc, "/pvc/disk.img", md5PrefixSize)
 		Expect(err).ToNot(HaveOccurred())
 		Expect(md5).To(Equal(utils.TinyCoreMD5))
+
+		By("Verify preallocated size")
+		ok, err := f.VerifyImagePreallocated(f.Namespace, pvc)
+		Expect(err).ToNot(HaveOccurred())
+		Expect(ok).To(BeFalse())
 	})
 
 	DescribeTable("All import paths should contain Preallocation step", func(shouldPreallocate bool, expectedMD5, path string, dvFunc func() *cdiv1.DataVolume) {
@@ -1047,6 +1066,13 @@ var _ = Describe("Preallocation", func() {
 			md5, err := f.GetMD5(f.Namespace, pvc, path, md5PrefixSize)
 			Expect(err).ToNot(HaveOccurred())
 			Expect(md5).To(Equal(expectedMD5))
+
+			if !f.IsBlockVolumeStorageClassAvailable() {
+				// Block volumes can't be read with qemu-img
+				ok, err := f.VerifyImagePreallocated(f.Namespace, pvc)
+				Expect(err).ToNot(HaveOccurred())
+				Expect(ok).To(BeTrue())
+			}
 		} else {
 			Expect(pvc.GetAnnotations()[controller.AnnPreallocationApplied]).ShouldNot(Equal("true"))
 		}
@@ -1101,6 +1127,36 @@ var _ = Describe("Preallocation", func() {
 			dataVolume.Spec.Source.Registry.CertConfigMap = cm
 			return dataVolume
 		}),
+		Entry("VddkImport", true, utils.VcenterMD5, "/pvc/disk.img", func() *cdiv1.DataVolume {
+			// Find vcenter-simulator pod
+			pod, err := utils.FindPodByPrefix(f.K8sClient, f.CdiInstallNs, "vcenter-deployment", "app=vcenter")
+			Expect(err).ToNot(HaveOccurred())
+			Expect(pod).ToNot(BeNil())
+
+			// Get test VM UUID
+			id, err := tests.RunKubectlCommand(f, "exec", "-n", pod.Namespace, pod.Name, "--", "cat", "/tmp/vmid")
+			Expect(err).To(BeNil())
+			vmid, err := uuid.Parse(strings.TrimSpace(id))
+			Expect(err).To(BeNil())
+
+			// Get disk name
+			disk, err := tests.RunKubectlCommand(f, "exec", "-n", pod.Namespace, pod.Name, "--", "cat", "/tmp/vmdisk")
+			Expect(err).To(BeNil())
+			disk = strings.TrimSpace(disk)
+			Expect(err).To(BeNil())
+
+			// Create VDDK login secret
+			stringData := map[string]string{
+				common.KeyAccess: "user",
+				common.KeySecret: "pass",
+			}
+			backingFile := disk
+			secretRef := "vddksecret"
+			thumbprint := "testprint"
+			s, _ := utils.CreateSecretFromDefinition(f.K8sClient, utils.NewSecretDefinition(nil, stringData, nil, f.Namespace.Name, secretRef))
+
+			return utils.NewDataVolumeWithVddkImport("import-dv", "100Mi", backingFile, s.Name, thumbprint, vcenterURL(), vmid.String())
+		}),
 		Entry("Blank image", true, utils.BlankMD5, "/pvc/disk.img", func() *cdiv1.DataVolume {
 			return utils.NewDataVolumeForBlankRawImage("import-dv", "100Mi")
 		}),
@@ -1112,4 +1168,38 @@ var _ = Describe("Preallocation", func() {
 			return utils.NewDataVolumeForBlankRawImageBlock("import-dv", "100Mi", f.BlockSCName)
 		}),
 	)
+
+	It("Filesystem overhead is honored with blank volume", func() {
+		tests.SetFilesystemOverhead(f, "0.055", "0.055")
+
+		dv := utils.NewDataVolumeForBlankRawImage("import-dv", "100Mi")
+		preallocation := true
+		dv.Spec.Preallocation = &preallocation
+		dataVolume, err = utils.CreateDataVolumeFromDefinition(f.CdiClient, f.Namespace.Name, dv)
+		Expect(err).ToNot(HaveOccurred())
+
+		pvc, err := utils.WaitForPVC(f.K8sClient, dataVolume.Namespace, dataVolume.Name)
+		Expect(err).ToNot(HaveOccurred())
+		f.ForceBindIfWaitForFirstConsumer(pvc)
+
+		By("Verify the pod status is succeeded on the target PVC")
+		found, err := utils.WaitPVCPodStatusSucceeded(f.K8sClient, pvc)
+		Expect(err).ToNot(HaveOccurred())
+		Expect(found).To(BeTrue())
+
+		Expect(f.VerifyFSOverhead(f.Namespace, pvc, preallocation)).To(BeTrue())
+
+		pvc, err = utils.FindPVC(f.K8sClient, dataVolume.Namespace, dataVolume.Name)
+		Expect(err).ToNot(HaveOccurred())
+		Expect(pvc.GetAnnotations()[controller.AnnPreallocationApplied]).Should(Equal("true"))
+
+		By("Verify content")
+		md5, err := f.GetMD5(f.Namespace, pvc, "/pvc/disk.img", md5PrefixSize)
+		Expect(err).ToNot(HaveOccurred())
+		Expect(md5).To(Equal(utils.BlankMD5))
+
+		ok, err := f.VerifyImagePreallocated(f.Namespace, pvc)
+		Expect(err).ToNot(HaveOccurred())
+		Expect(ok).To(BeTrue())
+	})
 })
